package com.sagacloud.extend.dao_data;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.sagacloud.util.compute_engine.core.AnnotationDescription;
import com.sagacloud.util.compute_engine.core.IDao_data;
import com.sagacloud.util.compute_engine.util.ConstantCE;
import com.sagacloud.util.compute_engine.util.DoubleTimeObject;
import com.sagacloud.util.compute_engine.util.QueryUtil;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@AnnotationDescription(value = "dao_data")
public class ImplDao_tenantbalance implements IDao_data {

    String database_name;
    String table_name;
    String dao_type = "tenantbalance";

    @Override
    public void Initialize(String database_name) {

        this.database_name = database_name;
        this.table_name = "data_" + dao_type;
    }

    @Override
    public String GetName() {
        return this.dao_type;
    }

    @Override
    public List<DoubleTimeObject> GetDataListGteLte(JSONObject dao_key, String time_period, Date time_from, Date time_to, boolean GteLte)
            throws Exception {
        List<DoubleTimeObject> result = null;

        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        String building = (String) dao_key.get("building");
        String tenant = (String) dao_key.get("tenant");
        String energy_type = (String) dao_key.get("energy_type");

        JSONObject Criteria = new JSONObject();
        Criteria.put("building", building);
        Criteria.put("tenant", tenant);
        Criteria.put("energy_type", energy_type);
        JSONObject range = new JSONObject();
        range.put("$gte", sdf.format(time_from));
        if (time_to != null) {
            if (GteLte) {
                range.put("$lte", sdf.format(time_to));
            } else {
                range.put("$lt", sdf.format(time_to));
            }
        }
        Criteria.put("data_time", range);
        String table_name_query = table_name;
        JSONArray resultContent = QueryUtil.select(database_name, table_name_query, Criteria);
        List<DoubleTimeObject> dataList = new ArrayList<DoubleTimeObject>();
        for (int ii = 0; ii < resultContent.size(); ii++) {
            JSONObject computelog = (JSONObject) resultContent.get(ii);
            DoubleTimeObject data = new DoubleTimeObject();
            data.data_time = sdf.parse((String) computelog.get("data_time"));
            data.data_value = (Double) computelog.get("data_value");
            dataList.add(data);
        }
        result = dataList;

        return result;
    }

    @Override
    public void InsertDataListGteLte(JSONObject dao_key, String time_period, List<DoubleTimeObject> outputDataList) throws Exception {
        if (ConstantCE.time_period_seperate_save_two || !ConstantCE.time_period_seperate) {
            this.InsertDataListGteLte(false, dao_key, time_period, outputDataList);
        }
        if (ConstantCE.time_period_seperate_save_two || ConstantCE.time_period_seperate) {
            this.InsertDataListGteLte(true, dao_key, time_period, outputDataList);
        }
    }

    @Override
    public void DeleteDataListGteLte(JSONObject dao_key, String time_period, Date time_from, Date time_to) throws Exception {
        if (ConstantCE.time_period_seperate_save_two || !ConstantCE.time_period_seperate) {
            this.DeleteDataListGteLte(false, dao_key, time_period, time_from, time_to);
        }
        if (ConstantCE.time_period_seperate_save_two || ConstantCE.time_period_seperate) {
            this.DeleteDataListGteLte(true, dao_key, time_period, time_from, time_to);
        }
    }

    private void DeleteDataListGteLte(boolean time_period_seperate, JSONObject dao_key, String time_period, Date time_from, Date time_to)
            throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

        String building = (String) dao_key.get("building");
        String tenant = (String) dao_key.get("tenant");
        String energy_type = (String) dao_key.get("energy_type");

        JSONObject Criteria = new JSONObject();
        Criteria.put("building", building);
        Criteria.put("tenant", tenant);
        Criteria.put("energy_type", energy_type);
        JSONObject range = new JSONObject();
        range.put("$gte", sdf.format(time_from));
        if (time_to != null) {
            range.put("$lte", sdf.format(time_to));
        }
        Criteria.put("data_time", range);
        String table_name_query = table_name;
        QueryUtil.delete(database_name, table_name_query, Criteria);
    }

    private void InsertDataListGteLte(boolean time_period_seperate, JSONObject dao_key, String time_period, List<DoubleTimeObject> outputDataList)
            throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

        String building = (String) dao_key.get("building");
        String tenant = (String) dao_key.get("tenant");
        String energy_type = (String) dao_key.get("energy_type");

        JSONArray InsertObjects = new JSONArray();
        for (DoubleTimeObject data : outputDataList) {
            JSONObject InsertObject = new JSONObject();
            InsertObject.put("building", building);
            InsertObject.put("tenant", tenant);
            InsertObject.put("energy_type", energy_type);
            InsertObject.put("data_time", sdf.format(data.data_time));
            InsertObject.put("data_value", data.data_value);
            InsertObjects.add(InsertObject);
        }
        String table_name_query = table_name;
        QueryUtil.batch_insert(database_name, table_name_query, InsertObjects);
    }

    public DoubleTimeObject GetData_last(JSONObject dao_key, Date time) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        String building = (String) dao_key.get("building");
        String tenant = (String) dao_key.get("tenant");
        String energy_type = (String) dao_key.get("energy_type");

        JSONObject Criteria = new JSONObject();
        Criteria.put("building", building);
        Criteria.put("tenant", tenant);
        Criteria.put("energy_type", energy_type);
        JSONObject range = new JSONObject();
        range.put("$lte", sdf.format(time));
        Criteria.put("data_time", range);
        JSONArray OrderBy = new JSONArray();
        OrderBy.add(QueryUtil.OrderByItem("building", true));
        OrderBy.add(QueryUtil.OrderByItem("tenant", true));
        OrderBy.add(QueryUtil.OrderByItem("energy_type", true));
        OrderBy.add(QueryUtil.OrderByItem("data_time", false));
        JSONObject Limit = new JSONObject();
        Limit.put("Skip", 0L);
        Limit.put("Count", 1L);
        String table_name_query = table_name;
        JSONArray resultContent = QueryUtil.select(database_name, table_name_query, Criteria, OrderBy, Limit);
        if (resultContent.size() > 0) {
            JSONObject computelog = (JSONObject) resultContent.get(0);
            DoubleTimeObject data = new DoubleTimeObject();
            data.data_time = sdf.parse((String) computelog.get("data_time"));
            data.data_value = (Double) computelog.get("data_value");
            return data;
        } else {
            return null;
        }
    }

    public DoubleTimeObject GetBalance_initial(JSONObject dao_key) throws Exception {
        String building = (String) dao_key.get("building");
        String tenant = (String) dao_key.get("tenant");
        String energy_type = (String) dao_key.get("energy_type");

        if (energy_type == null) {
            JSONObject Criteria = new JSONObject();
            Criteria.put("building", building);
            Criteria.put("sign", tenant);
            String table_name_query = "tenant";
            JSONArray resultContent = QueryUtil.select(database_name, table_name_query, Criteria);
            if (resultContent.size() > 0) {
                JSONObject computelog = (JSONObject) resultContent.get(0);
                DoubleTimeObject data = new DoubleTimeObject();
                data.data_time = null;
                data.data_value = (Double) computelog.get("soft_initial_yuer");
                return data;
            } else {
                return null;
            }
        } else {
            JSONObject Criteria = new JSONObject();
            Criteria.put("building", building);
            Criteria.put("sign", tenant);
            Criteria.put("energy_type", energy_type);
            String table_name_query = "tenant_energy";
            JSONArray resultContent = QueryUtil.select(database_name, table_name_query, Criteria);
            if (resultContent.size() > 0) {
                JSONObject computelog = (JSONObject) resultContent.get(0);
                DoubleTimeObject data = new DoubleTimeObject();
                data.data_time = null;
                data.data_value = (Double) computelog.get("soft_initial_yuer");
                return data;
            } else {
                return null;
            }
        }
    }

    public List<DoubleTimeObject> GetRechargeList(JSONObject dao_key, Date time) throws Exception {
        List<DoubleTimeObject> result = new ArrayList<DoubleTimeObject>();

        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        String building = (String) dao_key.get("building");
        String tenant = (String) dao_key.get("tenant");
        String energy_type = (String) dao_key.get("energy_type");

        JSONObject Criteria = new JSONObject();
        Criteria.put("building", building);
        Criteria.put("sign", tenant);
        Criteria.put("energy_type", energy_type);
        JSONObject range = new JSONObject();
        range.put("$gte", sdf.format(time));
        Criteria.put("operate_time", range);
        String table_name_query = "tenant_recharge";
        JSONArray resultContent = QueryUtil.select(database_name, table_name_query, Criteria);
        for (int i = 0; i < resultContent.size(); i++) {
            JSONObject computelog = (JSONObject) resultContent.get(i);
            DoubleTimeObject data = new DoubleTimeObject();
            data.data_time = sdf.parse((String) computelog.get("operate_time"));
            data.data_value = (Double) computelog.get("amount");
            result.add(data);
        }
        return result;
    }

    public void SetRecharge_used(JSONObject dao_key, Date time) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        String building = (String) dao_key.get("building");
        String tenant = (String) dao_key.get("tenant");
        String energy_type = (String) dao_key.get("energy_type");

        JSONObject Criteria = new JSONObject();
        Criteria.put("building", building);
        Criteria.put("sign", tenant);
        Criteria.put("energy_type", energy_type);
        Criteria.put("operate_time", sdf.format(time));
        String table_name_query = "tenant_recharge";
        JSONObject Set = new JSONObject();
        Set.put("is_used", 1L);
        QueryUtil.update(database_name, table_name_query, Criteria, Set);
    }

    public DoubleTimeObject GetBalance_present(JSONObject dao_key) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        String building = (String) dao_key.get("building");
        String tenant = (String) dao_key.get("tenant");
        String energy_type = (String) dao_key.get("energy_type");

        JSONObject Criteria = new JSONObject();
        Criteria.put("building", building);
        Criteria.put("tenant", tenant);
        Criteria.put("energy_type", energy_type);
        String table_name_query = "data_tenantbalance_present";
        JSONArray resultContent = QueryUtil.select(database_name, table_name_query, Criteria);
        if (resultContent.size() > 0) {
            JSONObject computelog = (JSONObject) resultContent.get(0);
            DoubleTimeObject data = new DoubleTimeObject();
            data.data_time = sdf.parse((String) computelog.get("data_time"));
            data.data_value = (Double) computelog.get("data_value");
            return data;
        } else {
            return null;
        }
    }

    public void InsertDataBalance_present(JSONObject dao_key, DoubleTimeObject data) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

        String building = (String) dao_key.get("building");
        String tenant = (String) dao_key.get("tenant");
        String energy_type = (String) dao_key.get("energy_type");

        JSONObject InsertObject = new JSONObject();
        InsertObject.put("building", building);
        InsertObject.put("tenant", tenant);
        InsertObject.put("energy_type", energy_type);
        InsertObject.put("data_time", sdf.format(data.data_time));
        InsertObject.put("data_value", data.data_value);
        String table_name_query = "data_tenantbalance_present";
        QueryUtil.insert(database_name, table_name_query, InsertObject);
    }

    public void UpdateBalance_present(JSONObject dao_key, DoubleTimeObject data) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

        String building = (String) dao_key.get("building");
        String tenant = (String) dao_key.get("tenant");
        String energy_type = (String) dao_key.get("energy_type");

        JSONObject Criteria = new JSONObject();
        Criteria.put("building", building);
        Criteria.put("tenant", tenant);
        Criteria.put("energy_type", energy_type);
        JSONObject Set = new JSONObject();
        Set.put("data_time", sdf.format(data.data_time));
        Set.put("data_value", data.data_value);
        String table_name_query = "data_tenantbalance_present";
        QueryUtil.update(database_name, table_name_query, Criteria, Set);
    }
}
