package com.sagacloud.util.compute_engine.ci;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.sagacloud.util.compute_engine.core.CoreInput;
import com.sagacloud.util.compute_engine.core.CoreInstance;
import com.sagacloud.util.compute_engine.core.CoreOutput;
import com.sagacloud.util.compute_engine.util.QueryUtil;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class Dao_ci {

    String database_name;

    public void Initialize(String database_name) {

        this.database_name = database_name;
    }

    public List<CoreInstance> GetList() throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        List<CoreInstance> result = new ArrayList<CoreInstance>();

        JSONObject Criteria = new JSONObject();
        JSONArray resultContent = QueryUtil.select(database_name, "ci_instance", Criteria);
        for (int ii = 0; ii < resultContent.size(); ii++) {
            JSONObject jsonItem = (JSONObject) resultContent.get(ii);
            CoreInstance data = new CoreInstance();
            data.object_sign = (String) jsonItem.get("object_sign");
            data.instance_name = (String) jsonItem.get("instance_name");
            result.add(data);
        }

        Criteria = new JSONObject();
        resultContent = QueryUtil.select(database_name, "ci_instanceinput", Criteria);
        for (int ii = 0; ii < resultContent.size(); ii++) {
            JSONObject jsonItem = (JSONObject) resultContent.get(ii);
            String object_sign = (String) jsonItem.get("object_sign");
            String instance_name = (String) jsonItem.get("instance_name");
            CoreInstance data = null;
            for (CoreInstance instance : result) {
                if (instance.object_sign.equals(object_sign) && instance.instance_name.equals(instance_name)) {
                    data = instance;
                    break;
                }
            }
            if (data != null) {
                CoreInput input = new CoreInput();
                input.input_sign = (String) jsonItem.get("input_sign");
                input.time_period = (String) jsonItem.get("time_period");
                input.data_class = (String) jsonItem.get("data_class");
                input.dao_type = (String) jsonItem.get("dao_type");
                input.dao_key = (JSONObject) JSON.parse((String) jsonItem.get("dao_key"));
                input.source_object_sign = (String) jsonItem.get("source_object_sign");
                input.source_instance_name = (String) jsonItem.get("source_instance_name");
                input.source_output_sign = (String) jsonItem.get("source_output_sign");
                data.inputList.add(input);
            }
        }

        Criteria = new JSONObject();
        resultContent = QueryUtil.select(database_name, "ci_instanceoutput", Criteria);
        for (int ii = 0; ii < resultContent.size(); ii++) {
            JSONObject jsonItem = (JSONObject) resultContent.get(ii);
            String object_sign = (String) jsonItem.get("object_sign");
            String instance_name = (String) jsonItem.get("instance_name");
            CoreInstance data = null;
            for (CoreInstance instance : result) {
                if (instance.object_sign.equals(object_sign) && instance.instance_name.equals(instance_name)) {
                    data = instance;
                    break;
                }
            }
            if (data != null) {
                CoreOutput output = new CoreOutput();
                output.output_sign = (String) jsonItem.get("output_sign");
                output.time_begin = sdf.parse((String) jsonItem.get("time_begin"));
                output.time_end = sdf.parse((String) jsonItem.get("time_end"));
                output.data_class = (String) jsonItem.get("data_class");
                output.dao_type = (String) jsonItem.get("dao_type");
                output.dao_key = (JSONObject) JSON.parse((String) jsonItem.get("dao_key"));
                output.need_all_var_data = (Boolean) jsonItem.get("need_all_var_data");
                output.time_period_intialized = (Boolean) jsonItem.get("time_period_intialized");
                output.time_period = (String) jsonItem.get("time_period");
                output.mode = (String) jsonItem.get("mode");
                if (jsonItem.containsKey("expression")) {
                    Object itemObject = jsonItem.get("expression");
                    if (itemObject != null) {
                        output.expression = (String) itemObject;
                    }
                }
                if (jsonItem.containsKey("custom_function_name")) {
                    Object itemObject = jsonItem.get("custom_function_name");
                    if (itemObject != null) {
                        output.custom_function_name = (String) itemObject;
                    }
                }
                if (jsonItem.containsKey("custom_paramsMap")) {
                    Object itemObject = jsonItem.get("custom_paramsMap");
                    if (itemObject != null) {
                        output.custom_paramsMap = new HashMap<String, String>();
                        JSONObject jsonColumn = (JSONObject) JSON.parse((String) itemObject);
                        for (String key : jsonColumn.keySet()) {
                            output.custom_paramsMap.put(key, (String) jsonColumn.get(key));
                        }
                    }
                }
                data.outputList.add(output);
            }
        }

        return result;
    }

    public void Insert(CoreInstance instance) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        {
            JSONObject InsertObject = new JSONObject();
            InsertObject.put("object_sign", instance.object_sign);
            InsertObject.put("instance_name", instance.instance_name);
            QueryUtil.insert(database_name, "ci_instance", InsertObject);
        }
        {
            JSONArray InsertObjects = new JSONArray();
            for (CoreInput data : instance.inputList) {
                JSONObject InsertObject = new JSONObject();
                InsertObject.put("object_sign", instance.object_sign);
                InsertObject.put("instance_name", instance.instance_name);
                InsertObject.put("input_sign", data.input_sign);
                InsertObject.put("time_period", data.time_period);
                InsertObject.put("data_class", data.data_class);
                InsertObject.put("dao_type", data.dao_type);
                InsertObject.put("dao_key", data.dao_key.toJSONString());
                InsertObject.put("source_object_sign", data.source_object_sign);
                InsertObject.put("source_instance_name", data.source_instance_name);
                InsertObject.put("source_output_sign", data.source_output_sign);
                InsertObjects.add(InsertObject);
            }
            QueryUtil.batch_insert(database_name, "ci_instanceinput", InsertObjects);
        }
        {
            JSONArray InsertObjects = new JSONArray();
            for (CoreOutput data : instance.outputList) {
                JSONObject InsertObject = new JSONObject();
                InsertObject.put("object_sign", instance.object_sign);
                InsertObject.put("instance_name", instance.instance_name);
                InsertObject.put("output_sign", data.output_sign);
                InsertObject.put("time_begin", sdf.format(data.time_begin));
                InsertObject.put("time_end", sdf.format(data.time_end));
                InsertObject.put("data_class", data.data_class);
                InsertObject.put("dao_type", data.dao_type);
                InsertObject.put("dao_key", data.dao_key.toJSONString());
                InsertObject.put("need_all_var_data", data.need_all_var_data);
                InsertObject.put("time_period_intialized", data.time_period_intialized);
                InsertObject.put("time_period", data.time_period);
                InsertObject.put("mode", data.mode);
                if (data.expression != null) {
                    InsertObject.put("expression", data.expression);
                }
                if (data.custom_function_name != null) {
                    InsertObject.put("custom_function_name", data.custom_function_name);
                }
                if (data.custom_paramsMap.size() > 0) {
                    JSONObject jsonColumn = new JSONObject();
                    for (String key : data.custom_paramsMap.keySet()) {
                        jsonColumn.put(key, data.custom_paramsMap.get(key));
                    }
                    InsertObject.put("custom_paramsMap", jsonColumn.toJSONString());
                }
                InsertObjects.add(InsertObject);
            }
            QueryUtil.batch_insert(database_name, "ci_instanceoutput", InsertObjects);
        }
    }

    public void Delete(String object_sign, String instance_name) throws Exception {
        JSONObject Criteria = new JSONObject();
        Criteria.put("object_sign", object_sign);
        Criteria.put("instance_name", instance_name);
        QueryUtil.delete(database_name, "ci_instance", Criteria);
        QueryUtil.delete(database_name, "ci_instanceinput", Criteria);
        QueryUtil.delete(database_name, "ci_instanceoutput", Criteria);
    }
}
