package com.sagacloud.util.compute_engine.core;


import lombok.extern.slf4j.Slf4j;
import org.reflections.Reflections;

import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.*;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
@Slf4j
public class Dao_dataManager {
    private static final Dao_dataManager instance = new Dao_dataManager();
    Map<String, IDao_data> daoMap = new HashMap<String, IDao_data>();
    private String database_name;

    private Dao_dataManager() {
    }

    public static Dao_dataManager Instance() {
        return instance;
    }

    public void Initialize(String database_name) {

        this.database_name = database_name;

        this.LoadClassByReflections();
        this.LoadClassByJar();
    }

    private void LoadClassByReflections() {
        try {
            Reflections reflections = new Reflections("com.sagacloud.extend.dao_data");
            Set<Class<?>> annotated = reflections.getTypesAnnotatedWith(AnnotationDescription.class);
            Iterator<Class<?>> classIter = annotated.iterator();
            while (classIter.hasNext()) {
                Class<?> clazz = classIter.next();
                Class<?>[] interfaces = clazz.getInterfaces();
                boolean flag = false;
                for (int ii = 0; ii < interfaces.length; ii++) {
                    Class<?> inter = interfaces[ii];
                    if ("com.sagacloud.util.compute_engine.core.IDao_data".equals(inter.getName())) {
                        flag = true;
                    }
                }
                if (flag) {
                    add((IDao_data) clazz.newInstance());
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage(),e);
        }
    }

    private void LoadClassByJar() {
        try {
            String dirPath = System.getProperty("user.dir") + "/dao_data/";
            File dir = new File(dirPath);
            String[] files = new String[0];
            // wrapper
            if (!dir.exists()) {
                dirPath = System.getProperty("user.dir") + "/../lib/dao_data/";
                dir = new File(dirPath);
                if (dir.exists()) {
                    files = dir.list();
                }
            } else {
                files = dir.list();
            }
            URL[] urls = new URL[files.length];
            for (int i = 0; i < files.length; i++) {
                urls[i] = new URL("file:" + dirPath + files[i]);
            }
            URLClassLoader loader = new URLClassLoader(urls);
            for (int i = 0; i < files.length; i++) {
                File file = new File(dirPath + files[i]);
                if (file.isFile()) {
                    JarFile jar = new JarFile(file);

                    Enumeration<JarEntry> entries = jar.entries();
                    while (entries.hasMoreElements()) {
                        JarEntry entry = entries.nextElement();

                        if (entry.getName().endsWith(".class")) {
                            String name = entry.getName();
                            name = name.substring(0, name.length() - 6);
                            name = name.replaceAll("/", ".");
                            Class<?> clazz = loader.loadClass(name);
                            Class<?>[] interfaces = clazz.getInterfaces();
                            boolean flag = false;
                            for (int ii = 0; ii < interfaces.length; ii++) {
                                Class<?> inter = interfaces[ii];
                                if ("com.sagacloud.util.compute_engine.core.IDao_data".equals(inter.getName())) {
                                    flag = true;
                                }
                            }
                            if (flag) {
                                add((IDao_data) clazz.newInstance());
                            }
                        }
                    }

                    jar.close();
                }
            }
            loader.close();
        } catch (Exception e) {
            log.error(e.getMessage(),e);
        }
    }

    private void add(IDao_data function) {
        function.Initialize(database_name);
        daoMap.put(function.GetName(), function);
    }

    public IDao_data Get(String dao_type) {
        return daoMap.get(dao_type);
    }
}
